@@ -9,19 +9,28 @@ from __future__ import absolute_import
 
 import logging
 import os
+import platform
 import sys
 
 import click
 import kaptan
 from click.exceptions import FileError
 
-from libtmux.common import has_gte_version, has_minimum_version, which
+from libtmux.common import (
+    has_gte_version,
+    has_minimum_version,
+    which,
+    get_version,
+    tmux_cmd,
+)
 from libtmux.exc import TmuxCommandNotFound
 from libtmux.server import Server
 
-from . import config, exc, log, util
+from libtmux import __version__ as libtmux_version
+
+from . import config, exc, log, util, __file__ as tmuxp_path
 from .__about__ import __version__
-from ._compat import string_types
+from ._compat import PY3, PYMINOR, string_types
 from .workspacebuilder import WorkspaceBuilder, freeze
 
 logger = logging.getLogger(__name__)
@@ -322,7 +331,7 @@ def scan_config(config, config_dir=None):
         config = normpath(join(cwd, config))
 
     # no extension, scan
-    if not splitext(config)[1]:
+    if path.isdir(config) or not splitext(config)[1]:
         if is_name:
             candidates = [
                 x
@@ -661,6 +670,102 @@ def startup(config_dir):
         os.makedirs(config_dir)
 
 
+@cli.command(name='shell')
+@click.argument('session_name', nargs=1, required=False)
+@click.argument('window_name', nargs=1, required=False)
+@click.option('-S', 'socket_path', help='pass-through for tmux -S')
+@click.option('-L', 'socket_name', help='pass-through for tmux -L')
+@click.option(
+    '-c',
+    'command',
+    help='Instead of opening shell, execute python code in libtmux and exit',
+)
+@click.option(
+    '--best',
+    'shell',
+    flag_value='best',
+    help='Use best shell available in site packages',
+    default=True,
+)
+@click.option('--pdb', 'shell', flag_value='pdb', help='Use plain pdb')
+@click.option(
+    '--code', 'shell', flag_value='code', help='Use stdlib\'s code.interact()'
+)
+@click.option(
+    '--ptipython', 'shell', flag_value='ptipython', help='Use ptpython + ipython'
+)
+@click.option('--ptpython', 'shell', flag_value='ptpython', help='Use ptpython')
+@click.option('--ipython', 'shell', flag_value='ipython', help='Use ipython')
+@click.option('--bpython', 'shell', flag_value='bpython', help='Use bpython')
+@click.option(
+    '--use-pythonrc/--no-startup',
+    'use_pythonrc',
+    help='Load PYTHONSTARTUP env var and ~/.pythonrc.py script in --code',
+    default=False,
+)
+@click.option(
+    '--use-vi-mode/--no-vi-mode',
+    'use_vi_mode',
+    help='Use vi-mode in ptpython/ptipython',
+    default=False,
+)
+def command_shell(
+    session_name,
+    window_name,
+    socket_name,
+    socket_path,
+    command,
+    shell,
+    use_pythonrc,
+    use_vi_mode,
+):
+    """Launch python shell for tmux server, session, window and pane.
+
+    Priority given to loaded session/wndow/pane objects:
+    - session_name and window_name arguments
+    - current shell: environmental variable of TMUX_PANE (which gives us window and
+      session)
+    - ``server.attached_session``, ``session.attached_window``, ``window.attached_pane``
+    """
+    server = Server(socket_name=socket_name, socket_path=socket_path)
+
+    util.raise_if_tmux_not_running(server=server)
+
+    current_pane = util.get_current_pane(server=server)
+
+    session = util.get_session(
+        server=server, session_name=session_name, current_pane=current_pane
+    )
+
+    window = util.get_window(
+        session=session, window_name=window_name, current_pane=current_pane
+    )
+
+    pane = util.get_pane(window=window, current_pane=current_pane)  # NOQA: F841
+
+    if command is not None:
+        exec(command)
+    else:
+        if shell == 'pdb' or (os.getenv('PYTHONBREAKPOINT') and PY3 and PYMINOR >= 7):
+            from ._compat import breakpoint as tmuxp_breakpoint
+
+            tmuxp_breakpoint()
+            return
+        else:
+            from .shell import launch
+
+            launch(
+                shell=shell,
+                use_pythonrc=use_pythonrc,  # shell: code
+                use_vi_mode=use_vi_mode,  # shell: ptpython, ptipython
+                # tmux environment / libtmux variables
+                server=server,
+                session=session,
+                window=window,
+                pane=pane,
+            )
+
+
 @cli.command(name='freeze')
 @click.argument('session_name', nargs=1, required=False)
 @click.option('-S', 'socket_path', help='pass-through for tmux -S')
@@ -703,7 +808,6 @@ def command_freeze(session_name, socket_name, socket_path, force):
     else:
         sys.exit('Unknown config format.')
 
-    print(newconfig)
     print(
         '---------------------------------------------------------------'
         '\n'
@@ -959,3 +1063,66 @@ def command_ls():
             if os.path.isdir(f) or ext not in VALID_CONFIG_DIR_FILE_EXTENSIONS:
                 continue
             print(stem)
+
+
+@cli.command(name='debug-info', short_help='Print out all diagnostic info')
+def command_debug_info():
+    """
+    Print debug info to submit with Issues.
+    """
+
+    def prepend_tab(strings):
+        """
+        Prepend tab to strings in list.
+        """
+        return list(map(lambda x: '\t%s' % x, strings))
+
+    def output_break():
+        """
+        Generate output break.
+        """
+        return '-' * 25
+
+    def format_tmux_resp(std_resp):
+        """
+        Format tmux command response for tmuxp stdout.
+        """
+        return '\n'.join(
+            [
+                '\n'.join(prepend_tab(std_resp.stdout)),
+                click.style('\n'.join(prepend_tab(std_resp.stderr)), fg='red'),
+            ]
+        )
+
+    output = [
+        output_break(),
+        'environment:\n%s'
+        % '\n'.join(
+            prepend_tab(
+                [
+                    'dist: %s' % platform.platform(),
+                    'arch: %s' % platform.machine(),
+                    'uname: %s' % '; '.join(platform.uname()[:3]),
+                    'version: %s' % platform.version(),
+                ]
+            )
+        ),
+        output_break(),
+        'python version: %s' % ' '.join(sys.version.split('\n')),
+        'system PATH: %s' % os.environ['PATH'],
+        'tmux version: %s' % get_version(),
+        'libtmux version: %s' % libtmux_version,
+        'tmuxp version: %s' % __version__,
+        'tmux path: %s' % which('tmux'),
+        'tmuxp path: %s' % tmuxp_path,
+        'shell: %s' % os.environ['SHELL'],
+        output_break(),
+        'tmux sessions:\n%s' % format_tmux_resp(tmux_cmd('list-sessions')),
+        'tmux windows:\n%s' % format_tmux_resp(tmux_cmd('list-windows')),
+        'tmux panes:\n%s' % format_tmux_resp(tmux_cmd('list-panes')),
+        'tmux global options:\n%s' % format_tmux_resp(tmux_cmd('show-options', '-g')),
+        'tmux window options:\n%s'
+        % format_tmux_resp(tmux_cmd('show-window-options', '-g')),
+    ]
+
+    click.echo('\n'.join(output))
